;; Routines to Call wdiff and use it from Emacs.
;; copyright 2003 Junichi Uekawa.

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; readme-debian.el is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with your Debian installation, in /usr/share/common-licenses/GPL
;; If not, write to the Free Software Foundation, 675 Mass Ave,
;; Cambridge, MA 02139, USA.

(defgroup wdiff nil "wdiff mode"
  :group 'tools
  :prefix "wdiff-")

(defun wdiff (file1 file2)
  "Obtain the wdiff of two files."
  (interactive "fFile1: \nfFile2: ")

  (let* ((wdiff-buffer (concat "*wdiff*" default-directory))
	 (wdiff-process (concat "wdiff-process-" default-directory)))
    (switch-to-buffer wdiff-buffer)
    (wdiff-mode)
    (kill-region (point-min) (point-max))
    (start-process wdiff-process wdiff-buffer "/usr/bin/wdiff" 
		   (expand-file-name file1) 
		   (expand-file-name file2))))

(defun wdiff-mode ()
  "Mode to view debc output.
\\{wdiff-mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'wdiff-mode)
  (setq mode-name "wdiff")
  (mapcar 'make-local-variable '(font-lock-defaults))
  (use-local-map wdiff-mode-map)
  (set-syntax-table wdiff-mode-syntax-table)
  (setq font-lock-defaults 
	'(
					;keywords start here
	  (("\\[-.*?-\\]" . font-lock-string-face)
	   ("{[+].*?[+]}" . font-lock-keyword-face)
	   )
	  nil		;keywords-only
	  nil		;case-fold
	  ()		;syntax-alist
	  ))
  (run-hooks 'wdiff-mode-hook)
)

(defvar wdiff-mode-map nil "Keymap for wdiff mode.")
(defvar wdiff-mode-syntax-table nil "Syntax table for wdiff mode.")
(if wdiff-mode-syntax-table
         ()              ; Do not change the table if it is already set up.
       (setq wdiff-mode-syntax-table (make-syntax-table))
       (modify-syntax-entry ?\" ".   " wdiff-mode-syntax-table)
       (modify-syntax-entry ?\\ ".   " wdiff-mode-syntax-table)
       (modify-syntax-entry ?' "w   " wdiff-mode-syntax-table))


(provide 'wdiff)
